import collections.abc
import enum
import nanobind
import os
import pydrake.common
import pydrake.geometry
import pydrake.math
import pydrake.multibody.plant
import pydrake.multibody.tree
import pydrake.solvers
import pydrake.symbolic
from _typeshed import Incomplete
from pydrake.common.cpp_template import CSpaceSeparatingPlane_ as CSpaceSeparatingPlane_
from typing import Any, Callable, ClassVar, overload

CheckIfSatisfiesConvexityRadius: nanobind.nb_func
ComputePairwiseIntersections: nanobind.nb_func
Iris: nanobind.nb_func
IrisNp: nanobind.nb_func
LoadIrisRegionsYamlFile: nanobind.nb_func
MakeIrisObstacles: nanobind.nb_func
PartitionConvexSet: nanobind.nb_func
SaveIrisRegionsYamlFile: nanobind.nb_func

class AffineBall(ConvexSet):
    MakeAffineBallFromLineSegment: ClassVar[nanobind.nb_func] = ...
    MakeAxisAligned: ClassVar[nanobind.nb_func] = ...
    MakeHypersphere: ClassVar[nanobind.nb_func] = ...
    MakeUnitBall: ClassVar[nanobind.nb_func] = ...
    MinimumVolumeCircumscribedEllipsoid: ClassVar[nanobind.nb_func] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, B, center) -> None: ...
    @overload
    def __init__(self, ellipsoid: Hyperellipsoid) -> None: ...
    def B(self, *args, **kwargs): ...
    def Clone(self) -> ConvexSet: ...
    def center(self, *args, **kwargs): ...
    def __copy__(self) -> ConvexSet: ...
    def __deepcopy__(self, arg: dict) -> ConvexSet: ...

class AffineSubspace(ConvexSet):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, basis, translation) -> None: ...
    @overload
    def __init__(self, set: ConvexSet, tol: float = ...) -> None: ...
    def AffineDimension(self) -> int: ...
    def Clone(self) -> ConvexSet: ...
    def ContainedIn(self, other: AffineSubspace, tol: float = ...) -> bool: ...
    def IsNearlyEqualTo(self, other: AffineSubspace, tol: float = ...) -> bool: ...
    def OrthogonalComplementBasis(self, *args, **kwargs): ...
    def ToGlobalCoordinates(self, *args, **kwargs): ...
    def ToLocalCoordinates(self, x) -> Any: ...
    def basis(self, *args, **kwargs): ...
    def translation(self, *args, **kwargs): ...
    def __copy__(self) -> ConvexSet: ...
    def __deepcopy__(self, arg: dict) -> ConvexSet: ...

class CIrisCollisionGeometry:
    def __init__(self, *args, **kwargs) -> None: ...
    def X_BG(self) -> pydrake.math.RigidTransform: ...
    def body_index(self) -> pydrake.multibody.tree.BodyIndex: ...
    def geometry(self) -> pydrake.geometry.Shape: ...
    def id(self) -> pydrake.geometry.GeometryId: ...
    def num_rationals(self) -> int: ...
    def type(self) -> CIrisGeometryType: ...

class CIrisGeometryType(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kCapsule: ClassVar[CIrisGeometryType] = ...
    kCylinder: ClassVar[CIrisGeometryType] = ...
    kPolytope: ClassVar[CIrisGeometryType] = ...
    kSphere: ClassVar[CIrisGeometryType] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class CSpaceSeparatingPlane:
    def __init__(self, *args, **kwargs) -> None: ...
    def __copy__(self) -> CSpaceSeparatingPlane: ...
    def __deepcopy__(self, arg: dict) -> CSpaceSeparatingPlane: ...
    @property
    def a(self): ...
    @property
    def b(self): ...
    @property
    def decision_variables(self): ...
    @property
    def expressed_body(self): ...
    @property
    def negative_side_geometry(self): ...
    @property
    def plane_degree(self): ...
    @property
    def positive_side_geometry(self): ...

class CSpaceSeparatingPlane_𝓣Variable𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, *args, **kwargs) -> None: ...
    def __copy__(self) -> CSpaceSeparatingPlane_TVariableU: ...
    def __deepcopy__(self, arg: dict) -> CSpaceSeparatingPlane_TVariableU: ...
    @property
    def a(self): ...
    @property
    def b(self): ...
    @property
    def decision_variables(self): ...
    @property
    def expressed_body(self): ...
    @property
    def negative_side_geometry(self): ...
    @property
    def plane_degree(self): ...
    @property
    def positive_side_geometry(self): ...

class CartesianProduct(ConvexSet):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, sets: collections.abc.Sequence[ConvexSet]) -> None: ...
    @overload
    def __init__(self, setA: ConvexSet, setB: ConvexSet) -> None: ...
    @overload
    def __init__(self, sets: collections.abc.Sequence[ConvexSet], A, b) -> None: ...
    @overload
    def __init__(self, query_object: pydrake.geometry.QueryObject, geometry_id: pydrake.geometry.GeometryId, reference_frame: pydrake.geometry.FrameId | None = ...) -> None: ...
    def A(self, *args, **kwargs): ...
    def b(self, *args, **kwargs): ...
    def factor(self, index: int) -> ConvexSet: ...
    def num_factors(self) -> int: ...

class ConvexHull(ConvexSet):
    def __init__(self, sets: collections.abc.Sequence[ConvexSet], remove_empty_sets: bool = ...) -> None: ...
    def element(self, index: int) -> ConvexSet: ...
    def empty_sets_removed(self) -> bool: ...
    def num_elements(self) -> int: ...
    def participating_sets(self) -> object: ...
    def sets(self) -> object: ...

class ConvexSet:
    def __init__(self, *args, **kwargs) -> None: ...
    @overload
    def AddPointInNonnegativeScalingConstraints(self, prog: pydrake.solvers.MathematicalProgram, x, t: pydrake.symbolic.Variable) -> list[pydrake.solvers.BindingTConstraintU]: ...
    @overload
    def AddPointInNonnegativeScalingConstraints(self, prog: pydrake.solvers.MathematicalProgram, A, b, c, d: float, x, t) -> list[pydrake.solvers.BindingTConstraintU]: ...
    def AddPointInSetConstraints(self, *args, **kwargs): ...
    def CalcVolume(self) -> float: ...
    def CalcVolumeViaSampling(self, generator: pydrake.common.RandomGenerator, desired_rel_accuracy: float = ..., max_num_samples: int = ...) -> SampledVolume: ...
    def Clone(self) -> ConvexSet: ...
    def IntersectsWith(self, other: ConvexSet) -> bool: ...
    def IsBounded(self, parallelism: pydrake.common.Parallelism = ...) -> bool: ...
    def IsEmpty(self) -> bool: ...
    def MaybeGetFeasiblePoint(self, *args, **kwargs): ...
    def MaybeGetPoint(self, *args, **kwargs): ...
    def PointInSet(self, x, tol: float = ...) -> bool: ...
    def Projection(self, *args, **kwargs): ...
    def ToShapeWithPose(self) -> tuple[pydrake.geometry.Shape, pydrake.math.RigidTransform]: ...
    @overload
    def ambient_dimension(self) -> int: ...
    @overload
    def ambient_dimension(self) -> Any: ...

class CspaceFreePolytope(CspaceFreePolytopeBase):
    class BilinearAlternationOptions:
        convergence_tol: float
        ellipsoid_scaling: float
        find_polytope_options: Incomplete
        max_iter: int
        def __init__(self) -> None: ...
        @property
        def find_lagrangian_options(self): ...

    class BinarySearchOptions:
        convergence_tol: Incomplete
        max_iter: Incomplete
        scale_max: Incomplete
        scale_min: Incomplete
        def __init__(self) -> None: ...
        @property
        def find_lagrangian_options(self): ...

    class EllipsoidMarginCost(enum.Enum):
        __new__: ClassVar[Callable] = ...
        _generate_next_value_: ClassVar[Callable] = ...
        _hashable_values_: ClassVar[list] = ...
        _member_map_: ClassVar[dict] = ...
        _member_names_: ClassVar[list] = ...
        _member_type_: ClassVar[type[object]] = ...
        _unhashable_values_: ClassVar[list] = ...
        _unhashable_values_map_: ClassVar[dict] = ...
        _use_args_: ClassVar[bool] = ...
        _value2member_map_: ClassVar[dict] = ...
        _value_repr_: ClassVar[None] = ...
        kGeometricMean: ClassVar[CspaceFreePolytope.EllipsoidMarginCost] = ...
        kSum: ClassVar[CspaceFreePolytope.EllipsoidMarginCost] = ...
        __nb_enum__: ClassVar[PyCapsule] = ...

    class FindPolytopeGivenLagrangianOptions:
        backoff_scale: Incomplete
        ellipsoid_margin_cost: Incomplete
        ellipsoid_margin_epsilon: Incomplete
        s_inner_pts: Incomplete
        search_s_bounds_lagrangians: Incomplete
        solver_id: Incomplete
        solver_options: Incomplete
        def __init__(self) -> None: ...

    class FindSeparationCertificateGivenPolytopeOptions(FindSeparationCertificateOptions):
        ignore_redundant_C: Incomplete
        def __init__(self) -> None: ...

    class SearchResult:
        def __init__(self) -> None: ...
        def C(self, *args, **kwargs): ...
        def a(self, *args, **kwargs): ...
        def b(self) -> dict[int, pydrake.symbolic.Polynomial]: ...
        def certified_polytope(self) -> HPolyhedron: ...
        def d(self, *args, **kwargs): ...
        def num_iter(self) -> int: ...

    class SeparatingPlaneLagrangians:
        def __init__(self, C_rows: int, s_size: int) -> None: ...
        def GetSolution(self, result: pydrake.solvers.MathematicalProgramResult) -> CspaceFreePolytope.SeparatingPlaneLagrangians: ...
        def polytope(self, *args, **kwargs): ...
        def s_lower(self, *args, **kwargs): ...
        def s_upper(self, *args, **kwargs): ...

    class SeparationCertificate:
        negative_side_rational_lagrangians: Incomplete
        positive_side_rational_lagrangians: Incomplete
        def __init__(self, *args, **kwargs) -> None: ...
        def GetSolution(self, plane_index: int, a, b: pydrake.symbolic.Polynomial, plane_decision_vars, result: pydrake.solvers.MathematicalProgramResult) -> CspaceFreePolytope.SeparationCertificateResult: ...

    class SeparationCertificateProgram(SeparationCertificateProgramBase):
        def __init__(self) -> None: ...
        @property
        def certificate(self): ...
        @property
        def plane_index(self): ...

    class SeparationCertificateResult:
        def __init__(self, *args, **kwargs) -> None: ...
        @property
        def a(self): ...
        @property
        def b(self): ...
        @property
        def negative_side_rational_lagrangians(self): ...
        @property
        def plane_decision_var_vals(self): ...
        @property
        def plane_index(self): ...
        @property
        def positive_side_rational_lagrangians(self): ...
        @property
        def result(self): ...
    def __init__(self, plant: pydrake.multibody.plant.MultibodyPlant, scene_graph: pydrake.geometry.SceneGraph, plane_order: SeparatingPlaneOrder, q_star, options: CspaceFreePolytopeBase.Options = ...) -> None: ...
    def BinarySearch(self, ignored_collision_pairs: collections.abc.Set[tuple[pydrake.geometry.GeometryId, pydrake.geometry.GeometryId]], C, d, s_center, options: CspaceFreePolytope.BinarySearchOptions) -> CspaceFreePolytope.SearchResult | None: ...
    def FindSeparationCertificateGivenPolytope(self, C, d, ignored_collision_pairs: collections.abc.Set[tuple[pydrake.geometry.GeometryId, pydrake.geometry.GeometryId]], options: CspaceFreePolytope.FindSeparationCertificateGivenPolytopeOptions) -> tuple[bool, dict[tuple[pydrake.geometry.GeometryId, pydrake.geometry.GeometryId], CspaceFreePolytope.SeparationCertificateResult]]: ...
    def MakeIsGeometrySeparableProgram(self, geometry_pair: tuple[pydrake.geometry.GeometryId, pydrake.geometry.GeometryId], C, d) -> CspaceFreePolytope.SeparationCertificateProgram: ...
    def SearchWithBilinearAlternation(self, ignored_collision_pairs: collections.abc.Set[tuple[pydrake.geometry.GeometryId, pydrake.geometry.GeometryId]], C_init, d_init, options: CspaceFreePolytope.BilinearAlternationOptions) -> list[CspaceFreePolytope.SearchResult]: ...
    def SolveSeparationCertificateProgram(self, certificate_program: CspaceFreePolytope.SeparationCertificateProgram, options: CspaceFreePolytope.FindSeparationCertificateGivenPolytopeOptions) -> CspaceFreePolytope.SeparationCertificateResult | None: ...

class CspaceFreePolytopeBase:
    class Options:
        with_cross_y: bool
        def __init__(self) -> None: ...
    def __init__(self, *args, **kwargs) -> None: ...
    def map_geometries_to_separating_planes(self) -> dict[tuple[pydrake.geometry.GeometryId, pydrake.geometry.GeometryId], int]: ...
    def separating_planes(self) -> list[CSpaceSeparatingPlane_TVariableU]: ...
    def y_slack(self, *args, **kwargs): ...

class FindSeparationCertificateOptions:
    parallelism: Incomplete
    solver_id: Incomplete
    solver_options: Incomplete
    terminate_at_failure: Incomplete
    verbose: Incomplete
    def __init__(self) -> None: ...

class GcsGraphvizOptions:
    precision: int
    scientific: Any
    show_costs: bool
    show_flows: bool
    show_slacks: bool
    show_vars: bool
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> GcsGraphvizOptions: ...
    def __deepcopy__(self, arg: dict) -> GcsGraphvizOptions: ...
    @property
    def __fields__(self): ...

class GraphOfConvexSets:
    class Edge:
        def __init__(self, *args, **kwargs) -> None: ...
        @overload
        def AddConstraint(self, f: pydrake.symbolic.Formula, use_in_transcription: collections.abc.Set[GraphOfConvexSets.Transcription] = ...) -> pydrake.solvers.BindingTConstraintU: ...
        @overload
        def AddConstraint(self, binding: pydrake.solvers.BindingTConstraintU, use_in_transcription: collections.abc.Set[GraphOfConvexSets.Transcription] = ...) -> pydrake.solvers.BindingTConstraintU: ...
        @overload
        def AddCost(self, e: pydrake.symbolic.Expression, use_in_transcription: collections.abc.Set[GraphOfConvexSets.Transcription] = ...) -> pydrake.solvers.BindingTCostU: ...
        @overload
        def AddCost(self, binding: pydrake.solvers.BindingTCostU, use_in_transcription: collections.abc.Set[GraphOfConvexSets.Transcription] = ...) -> pydrake.solvers.BindingTCostU: ...
        def AddPhiConstraint(self, phi_value: bool) -> None: ...
        def ClearPhiConstraints(self) -> None: ...
        def GetConstraints(self, used_in_transcription: collections.abc.Set[GraphOfConvexSets.Transcription] = ...) -> list[pydrake.solvers.BindingTConstraintU]: ...
        def GetCosts(self, used_in_transcription: collections.abc.Set[GraphOfConvexSets.Transcription] = ...) -> list[pydrake.solvers.BindingTCostU]: ...
        @overload
        def GetSolutionCost(self, result: pydrake.solvers.MathematicalProgramResult) -> float | None: ...
        @overload
        def GetSolutionCost(self, result: pydrake.solvers.MathematicalProgramResult, cost: pydrake.solvers.BindingTCostU) -> float | None: ...
        def GetSolutionPhiXu(self, *args, **kwargs): ...
        def GetSolutionPhiXv(self, *args, **kwargs): ...
        def id(self) -> GraphOfConvexSets.EdgeId: ...
        def name(self) -> str: ...
        @overload
        def phi(self) -> pydrake.symbolic.Variable: ...
        @overload
        def phi(self) -> Any: ...
        def u(self) -> GraphOfConvexSets.Vertex: ...
        def v(self) -> GraphOfConvexSets.Vertex: ...
        def xu(self) -> Any: ...
        def xv(self) -> Any: ...

    class EdgeId:
        get_new_id: ClassVar[nanobind.nb_func] = ...
        def __init__(self, *args, **kwargs) -> None: ...
        def get_value(self) -> int: ...
        def is_valid(self) -> bool: ...
        def __eq__(self, arg: GraphOfConvexSets.EdgeId) -> bool: ...
        def __hash__(self) -> int: ...
        def __lt__(self, arg: GraphOfConvexSets.EdgeId) -> bool: ...
        def __ne__(self, arg: GraphOfConvexSets.EdgeId) -> bool: ...

    class Transcription(enum.Enum):
        __new__: ClassVar[Callable] = ...
        _generate_next_value_: ClassVar[Callable] = ...
        _hashable_values_: ClassVar[list] = ...
        _member_map_: ClassVar[dict] = ...
        _member_names_: ClassVar[list] = ...
        _member_type_: ClassVar[type[object]] = ...
        _unhashable_values_: ClassVar[list] = ...
        _unhashable_values_map_: ClassVar[dict] = ...
        _use_args_: ClassVar[bool] = ...
        _value2member_map_: ClassVar[dict] = ...
        _value_repr_: ClassVar[None] = ...
        kMIP: ClassVar[GraphOfConvexSets.Transcription] = ...
        kRelaxation: ClassVar[GraphOfConvexSets.Transcription] = ...
        kRestriction: ClassVar[GraphOfConvexSets.Transcription] = ...
        __nb_enum__: ClassVar[PyCapsule] = ...

    class Vertex:
        def __init__(self, *args, **kwargs) -> None: ...
        @overload
        def AddConstraint(self, f: pydrake.symbolic.Formula, use_in_transcription: collections.abc.Set[GraphOfConvexSets.Transcription] = ...) -> pydrake.solvers.BindingTConstraintU: ...
        @overload
        def AddConstraint(self, binding: pydrake.solvers.BindingTConstraintU, use_in_transcription: collections.abc.Set[GraphOfConvexSets.Transcription] = ...) -> pydrake.solvers.BindingTConstraintU: ...
        @overload
        def AddCost(self, e: pydrake.symbolic.Expression, use_in_transcription: collections.abc.Set[GraphOfConvexSets.Transcription] = ...) -> pydrake.solvers.BindingTCostU: ...
        @overload
        def AddCost(self, binding: pydrake.solvers.BindingTCostU, use_in_transcription: collections.abc.Set[GraphOfConvexSets.Transcription] = ...) -> pydrake.solvers.BindingTCostU: ...
        def GetConstraints(self, used_in_transcription: collections.abc.Set[GraphOfConvexSets.Transcription] = ...) -> list[pydrake.solvers.BindingTConstraintU]: ...
        def GetCosts(self, used_in_transcription: collections.abc.Set[GraphOfConvexSets.Transcription] = ...) -> list[pydrake.solvers.BindingTCostU]: ...
        def GetSolution(self, *args, **kwargs): ...
        @overload
        def GetSolutionCost(self, result: pydrake.solvers.MathematicalProgramResult) -> float | None: ...
        @overload
        def GetSolutionCost(self, result: pydrake.solvers.MathematicalProgramResult, cost: pydrake.solvers.BindingTCostU) -> float | None: ...
        def ambient_dimension(self) -> int: ...
        def id(self) -> GraphOfConvexSets.VertexId: ...
        def incoming_edges(self) -> list[GraphOfConvexSets.Edge]: ...
        def name(self) -> str: ...
        def outgoing_edges(self) -> list[GraphOfConvexSets.Edge]: ...
        def set(self) -> ConvexSet: ...
        def x(self, *args, **kwargs): ...

    class VertexId:
        get_new_id: ClassVar[nanobind.nb_func] = ...
        def __init__(self, *args, **kwargs) -> None: ...
        def get_value(self) -> int: ...
        def is_valid(self) -> bool: ...
        def __eq__(self, arg: GraphOfConvexSets.VertexId) -> bool: ...
        def __hash__(self) -> int: ...
        def __lt__(self, arg: GraphOfConvexSets.VertexId) -> bool: ...
        def __ne__(self, arg: GraphOfConvexSets.VertexId) -> bool: ...
    def __init__(self) -> None: ...
    def AddEdge(self, u: GraphOfConvexSets.Vertex, v: GraphOfConvexSets.Vertex, name: str = ...) -> GraphOfConvexSets.Edge: ...
    def AddEdgeFromTemplate(self, u: GraphOfConvexSets.Vertex, v: GraphOfConvexSets.Vertex, template_edge: GraphOfConvexSets.Edge) -> GraphOfConvexSets.Edge: ...
    def AddVertex(self, set: ConvexSet, name: str = ...) -> GraphOfConvexSets.Vertex: ...
    def AddVertexFromTemplate(self, template_vertex: GraphOfConvexSets.Vertex) -> GraphOfConvexSets.Vertex: ...
    def ClearAllPhiConstraints(self) -> None: ...
    def Clone(self) -> GraphOfConvexSets: ...
    def Edges(self) -> list[GraphOfConvexSets.Edge]: ...
    def GetEdgeByName(self, name: str) -> GraphOfConvexSets.Edge: ...
    @overload
    def GetGraphvizString(self, result: pydrake.solvers.MathematicalProgramResult | None = ..., options: GcsGraphvizOptions = ..., active_path: collections.abc.Sequence[GraphOfConvexSets.Edge] = ...) -> str: ...
    @overload
    def GetGraphvizString(self, result: pydrake.solvers.MathematicalProgramResult | None = ..., show_slacks: bool = ..., show_vars: bool = ..., show_flows: bool = ..., show_costs: bool = ..., scientific: bool = ..., precision: int = ..., active_path: collections.abc.Sequence[GraphOfConvexSets.Edge] = ...) -> str: ...
    def GetMutableEdgeByName(self, name: str) -> GraphOfConvexSets.Edge: ...
    def GetMutableVertexByName(self, name: str) -> GraphOfConvexSets.Vertex: ...
    def GetSolutionPath(self, source: GraphOfConvexSets.Vertex, target: GraphOfConvexSets.Vertex, result: pydrake.solvers.MathematicalProgramResult, tolerance: float = ...) -> list[GraphOfConvexSets.Edge]: ...
    def GetVertexByName(self, name: str) -> GraphOfConvexSets.Vertex: ...
    @overload
    def IsValid(self, v: GraphOfConvexSets.Vertex) -> bool: ...
    @overload
    def IsValid(self, e: GraphOfConvexSets.Edge) -> bool: ...
    def RemoveEdge(self, edge: GraphOfConvexSets.Edge) -> None: ...
    def RemoveVertex(self, vertex: GraphOfConvexSets.Vertex) -> None: ...
    @overload
    def SamplePaths(self, source: GraphOfConvexSets.Vertex, target: GraphOfConvexSets.Vertex, flows: collections.abc.Mapping[GraphOfConvexSets.Edge, float], options: GraphOfConvexSetsOptions) -> list[list[GraphOfConvexSets.Edge]]: ...
    @overload
    def SamplePaths(self, source: GraphOfConvexSets.Vertex, target: GraphOfConvexSets.Vertex, result: pydrake.solvers.MathematicalProgramResult, options: GraphOfConvexSetsOptions) -> list[list[GraphOfConvexSets.Edge]]: ...
    def SolveConvexRestriction(self, active_edges: collections.abc.Sequence[GraphOfConvexSets.Edge], options: GraphOfConvexSetsOptions = ..., initial_guess: pydrake.solvers.MathematicalProgramResult | None = ...) -> pydrake.solvers.MathematicalProgramResult: ...
    def SolveShortestPath(self, source: GraphOfConvexSets.Vertex, target: GraphOfConvexSets.Vertex, options: GraphOfConvexSetsOptions = ...) -> pydrake.solvers.MathematicalProgramResult: ...
    def Vertices(self) -> list[GraphOfConvexSets.Vertex]: ...
    def num_edges(self) -> int: ...
    def num_vertices(self) -> int: ...
    def __copy__(self) -> GraphOfConvexSets: ...
    def __deepcopy__(self, arg: dict) -> GraphOfConvexSets: ...

class GraphOfConvexSetsOptions:
    convex_relaxation: bool | None
    flow_tolerance: float
    max_rounded_paths: int | None
    max_rounding_trials: int
    parallelism: Incomplete
    preprocessing: bool | None
    preprocessing_solver: pydrake.solvers.SolverInterface
    preprocessing_solver_options: pydrake.solvers.SolverOptions | None
    restriction_solver: pydrake.solvers.SolverInterface
    restriction_solver_options: pydrake.solvers.SolverOptions | None
    rounding_seed: int
    solver: pydrake.solvers.SolverInterface
    solver_options: pydrake.solvers.SolverOptions
    def __init__(self) -> None: ...

class HPolyhedron(ConvexSet):
    MakeBox: ClassVar[nanobind.nb_func] = ...
    MakeL1Ball: ClassVar[nanobind.nb_func] = ...
    MakeUnitBox: ClassVar[nanobind.nb_func] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, A, b) -> None: ...
    @overload
    def __init__(self, query_object: pydrake.geometry.QueryObject, geometry_id: pydrake.geometry.GeometryId, reference_frame: pydrake.geometry.FrameId | None = ...) -> None: ...
    @overload
    def __init__(self, vpoly: VPolytope, tol: float = ...) -> None: ...
    @overload
    def __init__(self, prog: pydrake.solvers.MathematicalProgram) -> None: ...
    def A(self, *args, **kwargs): ...
    def CartesianPower(self, n: int) -> HPolyhedron: ...
    def CartesianProduct(self, other: HPolyhedron) -> HPolyhedron: ...
    def ChebyshevCenter(self, *args, **kwargs): ...
    def ContainedIn(self, other: HPolyhedron, tol: float = ...) -> bool: ...
    def FindRedundant(self, tol: float = ...) -> set[int]: ...
    def Intersection(self, other: HPolyhedron, check_for_redundancy: bool = ..., tol: float = ...) -> HPolyhedron: ...
    def MaximumVolumeInscribedAffineTransformation(self, circumbody: HPolyhedron, check_bounded: bool = ...) -> HPolyhedron: ...
    def MaximumVolumeInscribedEllipsoid(self) -> Hyperellipsoid: ...
    def PontryaginDifference(self, other: HPolyhedron) -> HPolyhedron: ...
    def ReduceInequalities(self, tol: float = ...) -> HPolyhedron: ...
    def Scale(self, scale: float, center=...) -> HPolyhedron: ...
    def SimplifyByIncrementalFaceTranslation(self, min_volume_ratio: float = ..., do_affine_transformation: bool = ..., max_iterations: int = ..., points_to_contain=..., intersecting_polytopes: collections.abc.Sequence[HPolyhedron] = ..., keep_whole_intersection: bool = ..., intersection_padding: float = ..., random_seed: int = ...) -> HPolyhedron: ...
    def UniformSample(self, *args, **kwargs): ...
    def b(self, *args, **kwargs): ...

class Hyperellipsoid(ConvexSet):
    MakeAxisAligned: ClassVar[nanobind.nb_func] = ...
    MakeHypersphere: ClassVar[nanobind.nb_func] = ...
    MakeUnitBall: ClassVar[nanobind.nb_func] = ...
    MinimumVolumeCircumscribedEllipsoid: ClassVar[nanobind.nb_func] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, A, center) -> None: ...
    @overload
    def __init__(self, query_object: pydrake.geometry.QueryObject, geometry_id: pydrake.geometry.GeometryId, reference_frame: pydrake.geometry.FrameId | None = ...) -> None: ...
    @overload
    def __init__(self, ellipsoid: AffineBall) -> None: ...
    def A(self, *args, **kwargs): ...
    def MinimumUniformScalingToTouch(self, *args, **kwargs): ...
    def center(self, *args, **kwargs): ...

class Hyperrectangle(ConvexSet):
    MaybeCalcAxisAlignedBoundingBox: ClassVar[nanobind.nb_func] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, lb, ub) -> None: ...
    def Center(self, *args, **kwargs): ...
    def MakeHPolyhedron(self) -> HPolyhedron: ...
    def MaybeGetIntersection(self, arg: Hyperrectangle) -> Hyperrectangle | None: ...
    def UniformSample(self, *args, **kwargs): ...
    def lb(self, *args, **kwargs): ...
    def ub(self, *args, **kwargs): ...

class ImplicitGraphOfConvexSets:
    def __init__(self) -> None: ...
    def ExpandRecursively(self, start: GraphOfConvexSets.Vertex, max_successor_calls: int = ...) -> None: ...
    def Successors(self, v: GraphOfConvexSets.Vertex) -> list[GraphOfConvexSets.Edge]: ...
    def gcs(self) -> GraphOfConvexSets: ...
    def mutable_gcs(self) -> GraphOfConvexSets: ...

class ImplicitGraphOfConvexSetsFromExplicit(ImplicitGraphOfConvexSets):
    def __init__(self, gcs: GraphOfConvexSets) -> None: ...
    def ImplicitVertexFromExplicit(self, v_explicit: GraphOfConvexSets.Vertex) -> GraphOfConvexSets.Vertex: ...

class Intersection(ConvexSet):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, sets: collections.abc.Sequence[ConvexSet]) -> None: ...
    @overload
    def __init__(self, setA: ConvexSet, setB: ConvexSet) -> None: ...
    def element(self, index: int) -> ConvexSet: ...
    def num_elements(self) -> int: ...

class IrisOptions:
    bounding_region: HPolyhedron | None
    configuration_obstacles: object
    configuration_space_margin: float
    iteration_limit: int
    mixing_steps: int
    num_additional_constraint_infeasible_samples: int
    num_collision_infeasible_samples: int
    prog_with_additional_constraints: pydrake.solvers.MathematicalProgram
    random_seed: int
    relative_termination_threshold: float
    require_sample_point_is_contained: bool
    solver_options: pydrake.solvers.SolverOptions | None
    starting_ellipse: Hyperellipsoid | None
    termination_threshold: float
    verify_domain_boundedness: Incomplete
    def __init__(self, **kwargs) -> None: ...

class MinkowskiSum(ConvexSet):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, sets: collections.abc.Sequence[ConvexSet]) -> None: ...
    @overload
    def __init__(self, setA: ConvexSet, setB: ConvexSet) -> None: ...
    @overload
    def __init__(self, query_object: pydrake.geometry.QueryObject, geometry_id: pydrake.geometry.GeometryId, reference_frame: pydrake.geometry.FrameId | None = ...) -> None: ...
    def num_terms(self) -> int: ...
    def term(self, index: int) -> ConvexSet: ...

class PlaneSide(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kNegative: ClassVar[PlaneSide] = ...
    kPositive: ClassVar[PlaneSide] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class Point(ConvexSet):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, x) -> None: ...
    @overload
    def __init__(self, query_object: pydrake.geometry.QueryObject, geometry_id: pydrake.geometry.GeometryId, reference_frame: pydrake.geometry.FrameId | None = ..., maximum_allowable_radius: float = ...) -> None: ...
    def set_x(self, x) -> None: ...
    def x(self, *args, **kwargs): ...

class SampledVolume:
    num_samples: int
    rel_accuracy: float
    volume: float
    def __init__(self, *args, **kwargs) -> None: ...

class SeparatingPlaneOrder(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kAffine: ClassVar[SeparatingPlaneOrder] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class SeparationCertificateProgramBase:
    def __init__(self, *args, **kwargs) -> None: ...
    def prog(self) -> pydrake.solvers.MathematicalProgram: ...
    @property
    def plane_index(self): ...

class SeparationCertificateResultBase:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def a(self): ...
    @property
    def b(self): ...
    @property
    def plane_decision_var_vals(self): ...
    @property
    def result(self): ...

class Spectrahedron(ConvexSet):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, prog: pydrake.solvers.MathematicalProgram) -> None: ...

class VPolytope(ConvexSet):
    MakeBox: ClassVar[nanobind.nb_func] = ...
    MakeUnitBox: ClassVar[nanobind.nb_func] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, vertices) -> None: ...
    @overload
    def __init__(self, H: HPolyhedron, tol: float = ...) -> None: ...
    @overload
    def __init__(self, query_object: pydrake.geometry.QueryObject, geometry_id: pydrake.geometry.GeometryId, reference_frame: pydrake.geometry.FrameId | None = ...) -> None: ...
    def GetMinimalRepresentation(self, tol: float = ...) -> VPolytope: ...
    def ToShapeConvex(self, convex_label: str = ...) -> pydrake.geometry.Convex: ...
    def WriteObj(self, filename: str | os.PathLike) -> None: ...
    def vertices(self, *args, **kwargs): ...

class _TemporaryName_N5drake8geometry12optimization21CSpaceSeparatingPlaneINS_8symbolic8VariableEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, *args, **kwargs) -> None: ...
    def __copy__(self) -> CSpaceSeparatingPlane_TVariableU: ...
    def __deepcopy__(self, arg: dict) -> CSpaceSeparatingPlane_TVariableU: ...
    @property
    def a(self): ...
    @property
    def b(self): ...
    @property
    def decision_variables(self): ...
    @property
    def expressed_body(self): ...
    @property
    def negative_side_geometry(self): ...
    @property
    def plane_degree(self): ...
    @property
    def positive_side_geometry(self): ...



import pydrake.autodiffutils
import pydrake.common
import pydrake.symbolic
from pydrake.common.cpp_template import Polynomial_ as Polynomial_
from typing import ClassVar, overload

class Polynomial:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg: float) -> None: ...
    @overload
    def __init__(self, coefficients) -> None: ...
    def CoefficientsAlmostEqual(self, other: Polynomial, tol: float = ..., tol_type: pydrake.common.ToleranceType = ...) -> bool: ...
    def Derivative(self, derivative_order: int = ...) -> Polynomial: ...
    def EvaluateUnivariate(self, x: float, derivative_order: int = ...) -> float: ...
    def GetCoefficients(self, *args, **kwargs): ...
    def GetDegree(self) -> int: ...
    def GetNumberOfCoefficients(self) -> int: ...
    def Integral(self, integration_constant: float = ...) -> Polynomial: ...
    def IsAffine(self) -> bool: ...
    def __add__(self, arg: Polynomial) -> Polynomial: ...
    def __eq__(self, arg: Polynomial) -> bool: ...
    def __mul__(self, arg: Polynomial) -> Polynomial: ...
    def __neg__(self) -> Polynomial: ...
    def __radd__(self, arg: float) -> Polynomial: ...
    def __rmul__(self, arg: float) -> Polynomial: ...
    def __rsub__(self, arg: float) -> Polynomial: ...
    def __sub__(self, arg: Polynomial) -> Polynomial: ...
    def __truediv__(self, arg: float) -> Polynomial: ...

class Polynomial_𝓣AutoDiffXd𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg: pydrake.autodiffutils.AutoDiffXd) -> None: ...
    @overload
    def __init__(self, coefficients) -> None: ...
    def CoefficientsAlmostEqual(self, other: Polynomial_TAutoDiffXdU, tol: pydrake.autodiffutils.AutoDiffXd = ..., tol_type: pydrake.common.ToleranceType = ...) -> bool: ...
    def Derivative(self, derivative_order: int = ...) -> Polynomial_TAutoDiffXdU: ...
    def EvaluateUnivariate(self, x: pydrake.autodiffutils.AutoDiffXd, derivative_order: int = ...) -> pydrake.autodiffutils.AutoDiffXd: ...
    def GetCoefficients(self, *args, **kwargs): ...
    def GetDegree(self) -> int: ...
    def GetNumberOfCoefficients(self) -> int: ...
    def Integral(self, integration_constant: pydrake.autodiffutils.AutoDiffXd = ...) -> Polynomial_TAutoDiffXdU: ...
    def IsAffine(self) -> bool: ...
    def __add__(self, arg: Polynomial_TAutoDiffXdU) -> Polynomial_TAutoDiffXdU: ...
    def __eq__(self, arg: Polynomial_TAutoDiffXdU) -> bool: ...
    def __mul__(self, arg: Polynomial_TAutoDiffXdU) -> Polynomial_TAutoDiffXdU: ...
    def __neg__(self) -> Polynomial_TAutoDiffXdU: ...
    def __radd__(self, arg: float) -> Polynomial_TAutoDiffXdU: ...
    def __rmul__(self, arg: float) -> Polynomial_TAutoDiffXdU: ...
    def __rsub__(self, arg: float) -> Polynomial_TAutoDiffXdU: ...
    def __sub__(self, arg: Polynomial_TAutoDiffXdU) -> Polynomial_TAutoDiffXdU: ...
    def __truediv__(self, arg: float) -> Polynomial_TAutoDiffXdU: ...

class Polynomial_𝓣Expression𝓤:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg: pydrake.symbolic.Expression) -> None: ...
    @overload
    def __init__(self, coefficients) -> None: ...
    def CoefficientsAlmostEqual(self, other: Polynomial_TExpressionU, tol: pydrake.symbolic.Expression = ..., tol_type: pydrake.common.ToleranceType = ...) -> pydrake.symbolic.Formula: ...
    def Derivative(self, derivative_order: int = ...) -> Polynomial_TExpressionU: ...
    def EvaluateUnivariate(self, x: pydrake.symbolic.Expression, derivative_order: int = ...) -> pydrake.symbolic.Expression: ...
    def GetCoefficients(self, *args, **kwargs): ...
    def GetDegree(self) -> int: ...
    def GetNumberOfCoefficients(self) -> int: ...
    def Integral(self, integration_constant: pydrake.symbolic.Expression = ...) -> Polynomial_TExpressionU: ...
    def IsAffine(self) -> bool: ...
    def __add__(self, arg: Polynomial_TExpressionU) -> Polynomial_TExpressionU: ...
    def __eq__(self, arg: Polynomial_TExpressionU) -> bool: ...
    def __mul__(self, arg: Polynomial_TExpressionU) -> Polynomial_TExpressionU: ...
    def __neg__(self) -> Polynomial_TExpressionU: ...
    def __radd__(self, arg: float) -> Polynomial_TExpressionU: ...
    def __rmul__(self, arg: float) -> Polynomial_TExpressionU: ...
    def __rsub__(self, arg: float) -> Polynomial_TExpressionU: ...
    def __sub__(self, arg: Polynomial_TExpressionU) -> Polynomial_TExpressionU: ...
    def __truediv__(self, arg: float) -> Polynomial_TExpressionU: ...

class _TemporaryName_N5drake10PolynomialINS_2ad8AutoDiffEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg: pydrake.autodiffutils.AutoDiffXd) -> None: ...
    @overload
    def __init__(self, coefficients) -> None: ...
    def CoefficientsAlmostEqual(self, other: Polynomial_TAutoDiffXdU, tol: pydrake.autodiffutils.AutoDiffXd = ..., tol_type: pydrake.common.ToleranceType = ...) -> bool: ...
    def Derivative(self, derivative_order: int = ...) -> Polynomial_TAutoDiffXdU: ...
    def EvaluateUnivariate(self, x: pydrake.autodiffutils.AutoDiffXd, derivative_order: int = ...) -> pydrake.autodiffutils.AutoDiffXd: ...
    def GetCoefficients(self, *args, **kwargs): ...
    def GetDegree(self) -> int: ...
    def GetNumberOfCoefficients(self) -> int: ...
    def Integral(self, integration_constant: pydrake.autodiffutils.AutoDiffXd = ...) -> Polynomial_TAutoDiffXdU: ...
    def IsAffine(self) -> bool: ...
    def __add__(self, arg: Polynomial_TAutoDiffXdU) -> Polynomial_TAutoDiffXdU: ...
    def __eq__(self, arg: Polynomial_TAutoDiffXdU) -> bool: ...
    def __mul__(self, arg: Polynomial_TAutoDiffXdU) -> Polynomial_TAutoDiffXdU: ...
    def __neg__(self) -> Polynomial_TAutoDiffXdU: ...
    def __radd__(self, arg: float) -> Polynomial_TAutoDiffXdU: ...
    def __rmul__(self, arg: float) -> Polynomial_TAutoDiffXdU: ...
    def __rsub__(self, arg: float) -> Polynomial_TAutoDiffXdU: ...
    def __sub__(self, arg: Polynomial_TAutoDiffXdU) -> Polynomial_TAutoDiffXdU: ...
    def __truediv__(self, arg: float) -> Polynomial_TAutoDiffXdU: ...

class _TemporaryName_N5drake10PolynomialINS_8symbolic10ExpressionEEE:
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg: pydrake.symbolic.Expression) -> None: ...
    @overload
    def __init__(self, coefficients) -> None: ...
    def CoefficientsAlmostEqual(self, other: Polynomial_TExpressionU, tol: pydrake.symbolic.Expression = ..., tol_type: pydrake.common.ToleranceType = ...) -> pydrake.symbolic.Formula: ...
    def Derivative(self, derivative_order: int = ...) -> Polynomial_TExpressionU: ...
    def EvaluateUnivariate(self, x: pydrake.symbolic.Expression, derivative_order: int = ...) -> pydrake.symbolic.Expression: ...
    def GetCoefficients(self, *args, **kwargs): ...
    def GetDegree(self) -> int: ...
    def GetNumberOfCoefficients(self) -> int: ...
    def Integral(self, integration_constant: pydrake.symbolic.Expression = ...) -> Polynomial_TExpressionU: ...
    def IsAffine(self) -> bool: ...
    def __add__(self, arg: Polynomial_TExpressionU) -> Polynomial_TExpressionU: ...
    def __eq__(self, arg: Polynomial_TExpressionU) -> bool: ...
    def __mul__(self, arg: Polynomial_TExpressionU) -> Polynomial_TExpressionU: ...
    def __neg__(self) -> Polynomial_TExpressionU: ...
    def __radd__(self, arg: float) -> Polynomial_TExpressionU: ...
    def __rmul__(self, arg: float) -> Polynomial_TExpressionU: ...
    def __rsub__(self, arg: float) -> Polynomial_TExpressionU: ...
    def __sub__(self, arg: Polynomial_TExpressionU) -> Polynomial_TExpressionU: ...
    def __truediv__(self, arg: float) -> Polynomial_TExpressionU: ...



"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""


from io import BytesIO
import struct

class lcmt_viewer_geometry_data(object):

    __slots__ = ["type", "position", "quaternion", "color", "string_data", "num_float_data", "float_data"]

    __typenames__ = ["int8_t", "float", "float", "float", "string", "int32_t", "float"]

    __dimensions__ = [None, [3], [4], [4], None, None, ["num_float_data"]]

    BOX = 1
    """ Defines an enum for geometry type. """
    SPHERE = 2
    CYLINDER = 3
    MESH = 4
    CAPSULE = 5
    ELLIPSOID = 6

    def __init__(self):
        self.type = 0
        """ LCM Type: int8_t """
        self.position = [ 0.0 for dim0 in range(3) ]
        """ LCM Type: float[3] """
        self.quaternion = [ 0.0 for dim0 in range(4) ]
        """
        x, y, z
        LCM Type: float[4]
        """

        self.color = [ 0.0 for dim0 in range(4) ]
        """
        w, x, y, z
        LCM Type: float[4]
        """

        self.string_data = ""
        """
        r, g, b, a
        If type == MESH there are different ways of encoding the mesh:
        
        - By path: string_data contains a file path to a mesh file.
        
        - Raw triangle mesh: a triangle mesh is wholly contained within the
        message. string_data will remain empty and the mesh is defined in
        float data as:
        
        V | T | v0 | v1 | ... vN | t0 | t1 | ... | tM
        
        where
        
        V: The number of vertices.
        T: The number of triangles.
        N: 3V, the number of floating point values for the V vertices.
        M: 3T, the number of vertex indices for the T triangles.
        
        - In-memory mesh: A json string stored in string_data which represents a
        C++ InMemoryMesh instance with only a single mesh file of the form:
        
        {
        "in_memory_mesh": {
        "mesh_file": {
        "contents": "VGhpcyBpcyBhIHRlc3Q=,
        "extension": ".txt",
        "filename_hint": "not_a_real.mesh"
        }
        }
        }
        
        or an in-memory mesh with supporting files of the form:
        
        {
        "in_memory_mesh": {
        "mesh_file": {
        "contents": "VGhpcyBpcyBhIHRlc3Q=,
        "extension": ".txt",
        "filename_hint": "not_a_real.mesh"
        },
        "supporting_files": [
        "in_memory.png": {
        "contents": "VGhpcyBpcyBhIHRlc3Q=,
        "extension": ".png",
        "filename_hint": "not_a_real.png"
        },
        "on_disk.png": {
        "path": "/file/to/image.png"
        },
        ]
        }
        }
        
        Note: Where file contents are provided, they are base64-encoded.
        
        LCM Type: string
        """

        self.num_float_data = 0
        """ LCM Type: int32_t """
        self.float_data = []
        """ LCM Type: float[num_float_data] """

    def encode(self):
        buf = BytesIO()
        buf.write(lcmt_viewer_geometry_data._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        buf.write(struct.pack(">b", self.type))
        buf.write(struct.pack('>3f', *self.position[:3]))
        buf.write(struct.pack('>4f', *self.quaternion[:4]))
        buf.write(struct.pack('>4f', *self.color[:4]))
        __string_data_encoded = self.string_data.encode('utf-8')
        buf.write(struct.pack('>I', len(__string_data_encoded)+1))
        buf.write(__string_data_encoded)
        buf.write(b"\0")
        buf.write(struct.pack(">i", self.num_float_data))
        buf.write(struct.pack('>%df' % self.num_float_data, *self.float_data[:self.num_float_data]))

    @staticmethod
    def decode(data: bytes):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != lcmt_viewer_geometry_data._get_packed_fingerprint():
            raise ValueError("Decode error")
        return lcmt_viewer_geometry_data._decode_one(buf)

    @staticmethod
    def _decode_one(buf):
        self = lcmt_viewer_geometry_data()
        self.type = struct.unpack(">b", buf.read(1))[0]
        self.position = struct.unpack('>3f', buf.read(12))
        self.quaternion = struct.unpack('>4f', buf.read(16))
        self.color = struct.unpack('>4f', buf.read(16))
        __string_data_len = struct.unpack('>I', buf.read(4))[0]
        self.string_data = buf.read(__string_data_len)[:-1].decode('utf-8', 'replace')
        self.num_float_data = struct.unpack(">i", buf.read(4))[0]
        self.float_data = struct.unpack('>%df' % self.num_float_data, buf.read(self.num_float_data * 4))
        return self

    @staticmethod
    def _get_hash_recursive(parents):
        if lcmt_viewer_geometry_data in parents: return 0
        tmphash = (0xae971a65992bed83) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _packed_fingerprint = None

    @staticmethod
    def _get_packed_fingerprint():
        if lcmt_viewer_geometry_data._packed_fingerprint is None:
            lcmt_viewer_geometry_data._packed_fingerprint = struct.pack(">Q", lcmt_viewer_geometry_data._get_hash_recursive([]))
        return lcmt_viewer_geometry_data._packed_fingerprint

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", lcmt_viewer_geometry_data._get_packed_fingerprint())[0]




import collections.abc
import enum
import nanobind
import numpy
import os
import pydrake.common.schema
import pydrake.geometry
import pydrake.math
import pydrake.multibody.tree
import pydrake.systems.framework
from _typeshed import Incomplete
from pydrake.common.cpp_template import Accelerometer_ as Accelerometer_, Gyroscope_ as Gyroscope_, Image as Image, ImageTraits as ImageTraits, RotaryEncoders_ as RotaryEncoders_
from typing import Any, Callable, ClassVar, overload

ApplyCameraConfig: nanobind.nb_func
ConvertDepth16UTo32F: nanobind.nb_func
ConvertDepth32FTo16U: nanobind.nb_func

class Accelerometer(pydrake.systems.framework.LeafSystem):
    def __init__(self, body: pydrake.multibody.tree.RigidBody, X_BS: pydrake.math.RigidTransform, gravity_vector=...) -> None: ...
    def body_index(self) -> pydrake.multibody.tree.BodyIndex: ...
    def get_body_accelerations_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_body_poses_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_body_velocities_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_measurement_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def gravity_vector(self, *args, **kwargs): ...
    def pose(self) -> pydrake.math.RigidTransform: ...

class Accelerometer_𝓣AutoDiffXd𝓤(pydrake.systems.framework.LeafSystem_TAutoDiffXdU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, body: pydrake.multibody.tree.RigidBody_TAutoDiffXdU, X_BS: pydrake.math.RigidTransform, gravity_vector=...) -> None: ...
    def body_index(self) -> pydrake.multibody.tree.BodyIndex: ...
    def get_body_accelerations_input_port(self) -> pydrake.systems.framework.InputPort_TAutoDiffXdU: ...
    def get_body_poses_input_port(self) -> pydrake.systems.framework.InputPort_TAutoDiffXdU: ...
    def get_body_velocities_input_port(self) -> pydrake.systems.framework.InputPort_TAutoDiffXdU: ...
    def get_measurement_output_port(self) -> pydrake.systems.framework.OutputPort_TAutoDiffXdU: ...
    def gravity_vector(self, *args, **kwargs): ...
    def pose(self) -> pydrake.math.RigidTransform: ...

class Accelerometer_𝓣Expression𝓤(pydrake.systems.framework.LeafSystem_TExpressionU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, body: pydrake.multibody.tree.RigidBody_TExpressionU, X_BS: pydrake.math.RigidTransform, gravity_vector=...) -> None: ...
    def body_index(self) -> pydrake.multibody.tree.BodyIndex: ...
    def get_body_accelerations_input_port(self) -> pydrake.systems.framework.InputPort_TExpressionU: ...
    def get_body_poses_input_port(self) -> pydrake.systems.framework.InputPort_TExpressionU: ...
    def get_body_velocities_input_port(self) -> pydrake.systems.framework.InputPort_TExpressionU: ...
    def get_measurement_output_port(self) -> pydrake.systems.framework.OutputPort_TExpressionU: ...
    def gravity_vector(self, *args, **kwargs): ...
    def pose(self) -> pydrake.math.RigidTransform: ...

class CameraConfig:
    class FocalLength:
        x: float | None
        y: float | None
        def __init__(self, **kwargs) -> None: ...
        def focal_x(self) -> float: ...
        def focal_y(self) -> float: ...
        def __copy__(self) -> CameraConfig.FocalLength: ...
        def __deepcopy__(self, arg: dict) -> CameraConfig.FocalLength: ...
        @property
        def __fields__(self): ...

    class FovDegrees:
        x: float | None
        y: float | None
        def __init__(self, **kwargs) -> None: ...
        def focal_x(self, width: int, height: int) -> float: ...
        def focal_y(self, width: int, height: int) -> float: ...
        def __copy__(self) -> CameraConfig.FovDegrees: ...
        def __deepcopy__(self, arg: dict) -> CameraConfig.FovDegrees: ...
        @property
        def __fields__(self): ...
    X_BC: pydrake.common.schema.Transform
    X_BD: pydrake.common.schema.Transform
    X_PB: pydrake.common.schema.Transform
    background: Incomplete
    capture_offset: float
    center_x: float | None
    center_y: float | None
    clipping_far: float
    clipping_near: float
    depth: bool
    do_compress: bool
    focal: CameraConfig.FocalLength | CameraConfig.FovDegrees
    fps: float
    height: int
    label: bool
    lcm_bus: str
    name: str
    output_delay: float
    renderer_class: str | pydrake.geometry.RenderEngineVtkParams | pydrake.geometry.RenderEngineGlParams | pydrake.geometry.RenderEngineGltfClientParams
    renderer_name: str
    rgb: bool
    show_rgb: Incomplete
    width: int
    z_far: float
    z_near: float
    def __init__(self, **kwargs) -> None: ...
    def MakeCameras(self) -> tuple[pydrake.geometry.ColorRenderCamera, pydrake.geometry.DepthRenderCamera]: ...
    def focal_x(self) -> float: ...
    def focal_y(self) -> float: ...
    def principal_point(self, *args, **kwargs): ...
    def __copy__(self) -> CameraConfig: ...
    def __deepcopy__(self, arg: dict) -> CameraConfig: ...
    @property
    def __fields__(self): ...

class CameraInfo:
    @overload
    def __init__(self, width: int, height: int, fov_y: float) -> None: ...
    @overload
    def __init__(self, width: int, height: int, intrinsic_matrix) -> None: ...
    @overload
    def __init__(self, width: int, height: int, focal_x: float, focal_y: float, center_x: float, center_y: float) -> None: ...
    def center_x(self) -> float: ...
    def center_y(self) -> float: ...
    def focal_x(self) -> float: ...
    def focal_y(self) -> float: ...
    def fov_x(self) -> float: ...
    def fov_y(self) -> float: ...
    def height(self) -> int: ...
    def intrinsic_matrix(self, *args, **kwargs): ...
    def width(self) -> int: ...

class Gyroscope(pydrake.systems.framework.LeafSystem):
    def __init__(self, body: pydrake.multibody.tree.RigidBody, X_BS: pydrake.math.RigidTransform) -> None: ...
    def body_index(self) -> pydrake.multibody.tree.BodyIndex: ...
    def get_body_poses_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_body_velocities_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_measurement_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def pose(self) -> pydrake.math.RigidTransform: ...

class Gyroscope_𝓣AutoDiffXd𝓤(pydrake.systems.framework.LeafSystem_TAutoDiffXdU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, body: pydrake.multibody.tree.RigidBody_TAutoDiffXdU, X_BS: pydrake.math.RigidTransform) -> None: ...
    def body_index(self) -> pydrake.multibody.tree.BodyIndex: ...
    def get_body_poses_input_port(self) -> pydrake.systems.framework.InputPort_TAutoDiffXdU: ...
    def get_body_velocities_input_port(self) -> pydrake.systems.framework.InputPort_TAutoDiffXdU: ...
    def get_measurement_output_port(self) -> pydrake.systems.framework.OutputPort_TAutoDiffXdU: ...
    def pose(self) -> pydrake.math.RigidTransform: ...

class Gyroscope_𝓣Expression𝓤(pydrake.systems.framework.LeafSystem_TExpressionU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, body: pydrake.multibody.tree.RigidBody_TExpressionU, X_BS: pydrake.math.RigidTransform) -> None: ...
    def body_index(self) -> pydrake.multibody.tree.BodyIndex: ...
    def get_body_poses_input_port(self) -> pydrake.systems.framework.InputPort_TExpressionU: ...
    def get_body_velocities_input_port(self) -> pydrake.systems.framework.InputPort_TExpressionU: ...
    def get_measurement_output_port(self) -> pydrake.systems.framework.OutputPort_TExpressionU: ...
    def pose(self) -> pydrake.math.RigidTransform: ...

class ImageBgr8U:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class ImageBgra8U:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class ImageDepth16U:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint16]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class ImageDepth32F:
    class Traits:
        ChannelType: ClassVar[type[numpy.float32]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: float) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class ImageFileFormat(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kJpeg: ClassVar[ImageFileFormat] = ...
    kPng: ClassVar[ImageFileFormat] = ...
    kTiff: ClassVar[ImageFileFormat] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class ImageGrey8U:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class ImageIo:
    class Metadata:
        channels: Incomplete
        depth: Incomplete
        format: Incomplete
        height: Incomplete
        scalar: Incomplete
        width: Incomplete
        def __init__(self, **kwargs) -> None: ...
        def __copy__(self) -> ImageIo.Metadata: ...
        def __deepcopy__(self, arg: dict) -> ImageIo.Metadata: ...
        @property
        def __fields__(self): ...
    def __init__(self) -> None: ...
    @overload
    def Load(self, path: str | os.PathLike, format: ImageFileFormat | None = ...) -> ImageTPixelTypepkBgr8UU | ImageTPixelTypepkBgra8UU | ImageTPixelTypepkDepth16UU | ImageTPixelTypepkDepth32FU | ImageTPixelTypepkGrey8UU | ImageTPixelTypepkRgb8UU | ImageTPixelTypepkRgba8UU | ImageTPixelTypepkLabel16IU: ...
    @overload
    def Load(self, buffer: bytes, format: ImageFileFormat | None = ...) -> ImageTPixelTypepkBgr8UU | ImageTPixelTypepkBgra8UU | ImageTPixelTypepkDepth16UU | ImageTPixelTypepkDepth32FU | ImageTPixelTypepkGrey8UU | ImageTPixelTypepkRgb8UU | ImageTPixelTypepkRgba8UU | ImageTPixelTypepkLabel16IU: ...
    @overload
    def LoadMetadata(self, path: str | os.PathLike) -> ImageIo.Metadata | None: ...
    @overload
    def LoadMetadata(self, buffer: bytes) -> ImageIo.Metadata | None: ...
    @overload
    def Save(self, image: ImageTPixelTypepkBgr8UU | ImageTPixelTypepkBgra8UU | ImageTPixelTypepkDepth16UU | ImageTPixelTypepkDepth32FU | ImageTPixelTypepkGrey8UU | ImageTPixelTypepkRgb8UU | ImageTPixelTypepkRgba8UU | ImageTPixelTypepkLabel16IU, path: str | os.PathLike, format: ImageFileFormat | None = ...) -> None: ...
    @overload
    def Save(self, image: ImageTPixelTypepkBgr8UU | ImageTPixelTypepkBgra8UU | ImageTPixelTypepkDepth16UU | ImageTPixelTypepkDepth32FU | ImageTPixelTypepkGrey8UU | ImageTPixelTypepkRgb8UU | ImageTPixelTypepkRgba8UU | ImageTPixelTypepkLabel16IU, format: ImageFileFormat) -> bytes: ...

class ImageLabel16I:
    class Traits:
        ChannelType: ClassVar[type[numpy.int16]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class ImageRgb8U:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class ImageRgba8U:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class ImageToLcmImageArrayT(pydrake.systems.framework.LeafSystem):
    DeclareImageInputPort𝓣PixelType𝓹kBgr8U𝓤: ClassVar[Callable] = ...
    DeclareImageInputPort𝓣PixelType𝓹kBgra8U𝓤: ClassVar[Callable] = ...
    DeclareImageInputPort𝓣PixelType𝓹kDepth16U𝓤: ClassVar[Callable] = ...
    DeclareImageInputPort𝓣PixelType𝓹kDepth32F𝓤: ClassVar[Callable] = ...
    DeclareImageInputPort𝓣PixelType𝓹kGrey8U𝓤: ClassVar[Callable] = ...
    DeclareImageInputPort𝓣PixelType𝓹kLabel16I𝓤: ClassVar[Callable] = ...
    DeclareImageInputPort𝓣PixelType𝓹kRgb8U𝓤: ClassVar[Callable] = ...
    DeclareImageInputPort𝓣PixelType𝓹kRgba8U𝓤: ClassVar[Callable] = ...
    DeclareImageInputPort: Incomplete
    @overload
    def __init__(self, color_frame_name: str, depth_frame_name: str, label_frame_name: str, do_compress: bool = ...) -> None: ...
    @overload
    def __init__(self, do_compress: bool = ...) -> None: ...
    def color_image_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def depth_image_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def image_array_t_msg_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def label_image_input_port(self) -> pydrake.systems.framework.InputPort: ...

class ImageTraits𝓣PixelType𝓹kBgr8U𝓤:
    ChannelType: ClassVar[type[numpy.uint8]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class ImageTraits𝓣PixelType𝓹kBgra8U𝓤:
    ChannelType: ClassVar[type[numpy.uint8]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class ImageTraits𝓣PixelType𝓹kDepth16U𝓤:
    ChannelType: ClassVar[type[numpy.uint16]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class ImageTraits𝓣PixelType𝓹kDepth32F𝓤:
    ChannelType: ClassVar[type[numpy.float32]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class ImageTraits𝓣PixelType𝓹kGrey8U𝓤:
    ChannelType: ClassVar[type[numpy.uint8]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class ImageTraits𝓣PixelType𝓹kLabel16I𝓤:
    ChannelType: ClassVar[type[numpy.int16]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class ImageTraits𝓣PixelType𝓹kRgb8U𝓤:
    ChannelType: ClassVar[type[numpy.uint8]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class ImageTraits𝓣PixelType𝓹kRgba8U𝓤:
    ChannelType: ClassVar[type[numpy.uint8]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class ImageWriter(pydrake.systems.framework.LeafSystem):
    def __init__(self) -> None: ...
    def DeclareImageInputPort(self, pixel_type: PixelType, port_name: str, file_name_format: str, publish_period: float, start_time: float) -> pydrake.systems.framework.InputPort: ...
    def ResetAllImageCounts(self) -> None: ...

class Image𝓣PixelType𝓹kBgr8U𝓤:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class Image𝓣PixelType𝓹kBgra8U𝓤:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class Image𝓣PixelType𝓹kDepth16U𝓤:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint16]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class Image𝓣PixelType𝓹kDepth32F𝓤:
    class Traits:
        ChannelType: ClassVar[type[numpy.float32]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: float) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class Image𝓣PixelType𝓹kGrey8U𝓤:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class Image𝓣PixelType𝓹kLabel16I𝓤:
    class Traits:
        ChannelType: ClassVar[type[numpy.int16]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class Image𝓣PixelType𝓹kRgb8U𝓤:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class Image𝓣PixelType𝓹kRgba8U𝓤:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class LcmImageArrayToImages(pydrake.systems.framework.LeafSystem):
    def __init__(self) -> None: ...
    def color_image_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def depth_image_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def image_array_t_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def label_image_output_port(self) -> pydrake.systems.framework.OutputPort: ...

class PixelFormat(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kBgr: ClassVar[PixelFormat] = ...
    kBgra: ClassVar[PixelFormat] = ...
    kDepth: ClassVar[PixelFormat] = ...
    kGrey: ClassVar[PixelFormat] = ...
    kLabel: ClassVar[PixelFormat] = ...
    kRgb: ClassVar[PixelFormat] = ...
    kRgba: ClassVar[PixelFormat] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class PixelScalar(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    k16I: ClassVar[PixelScalar] = ...
    k16U: ClassVar[PixelScalar] = ...
    k32F: ClassVar[PixelScalar] = ...
    k8U: ClassVar[PixelScalar] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class PixelType(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kBgr8U: ClassVar[PixelType] = ...
    kBgra8U: ClassVar[PixelType] = ...
    kDepth16U: ClassVar[PixelType] = ...
    kDepth32F: ClassVar[PixelType] = ...
    kGrey8U: ClassVar[PixelType] = ...
    kLabel16I: ClassVar[PixelType] = ...
    kRgb8U: ClassVar[PixelType] = ...
    kRgba8U: ClassVar[PixelType] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class RgbdSensor(pydrake.systems.framework.LeafSystem):
    @overload
    def __init__(self, parent_id: pydrake.geometry.FrameId, X_PB: pydrake.math.RigidTransform, color_camera: pydrake.geometry.ColorRenderCamera, depth_camera: pydrake.geometry.DepthRenderCamera) -> None: ...
    @overload
    def __init__(self, parent_id: pydrake.geometry.FrameId, X_PB: pydrake.math.RigidTransform, depth_camera: pydrake.geometry.DepthRenderCamera, show_window: bool = ...) -> None: ...
    def GetColorRenderCamera(self, context: pydrake.systems.framework.Context) -> pydrake.geometry.ColorRenderCamera: ...
    def GetDepthRenderCamera(self, context: pydrake.systems.framework.Context) -> pydrake.geometry.DepthRenderCamera: ...
    def GetParentFrameId(self, context: pydrake.systems.framework.Context) -> pydrake.geometry.FrameId: ...
    def GetX_PB(self, context: pydrake.systems.framework.Context) -> pydrake.math.RigidTransform: ...
    def SetColorRenderCamera(self, context: pydrake.systems.framework.Context, color_camera: pydrake.geometry.ColorRenderCamera) -> None: ...
    def SetDepthRenderCamera(self, context: pydrake.systems.framework.Context, depth_camera: pydrake.geometry.DepthRenderCamera) -> None: ...
    def SetParentFrameId(self, context: pydrake.systems.framework.Context, id: pydrake.geometry.FrameId) -> None: ...
    def SetX_PB(self, context: pydrake.systems.framework.Context, sensor_pose: pydrake.math.RigidTransform) -> None: ...
    def body_pose_in_world_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def color_image_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def default_X_PB(self) -> pydrake.math.RigidTransform: ...
    def default_color_render_camera(self) -> pydrake.geometry.ColorRenderCamera: ...
    def default_depth_render_camera(self) -> pydrake.geometry.DepthRenderCamera: ...
    def default_parent_frame_id(self) -> pydrake.geometry.FrameId: ...
    def depth_image_16U_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def depth_image_32F_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def image_time_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def label_image_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def query_object_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def set_default_X_PB(self, sensor_pose: pydrake.math.RigidTransform) -> None: ...
    def set_default_color_render_camera(self, color_camera: pydrake.geometry.ColorRenderCamera) -> None: ...
    def set_default_depth_render_camera(self, depth_camera: pydrake.geometry.DepthRenderCamera) -> None: ...
    def set_default_parent_frame_id(self, id: pydrake.geometry.FrameId) -> None: ...

class RgbdSensorAsync(pydrake.systems.framework.LeafSystem):
    def __init__(self, scene_graph: pydrake.geometry.SceneGraph, parent_id: pydrake.geometry.FrameId, X_PB: pydrake.math.RigidTransform, fps: float, capture_offset: float, output_delay: float, color_camera: pydrake.geometry.ColorRenderCamera | None, depth_camera: pydrake.geometry.DepthRenderCamera | None = ..., render_label_image: bool = ...) -> None: ...
    def GetColorRenderCamera(self, context: pydrake.systems.framework.Context) -> pydrake.geometry.ColorRenderCamera | None: ...
    def GetDepthRenderCamera(self, context: pydrake.systems.framework.Context) -> pydrake.geometry.DepthRenderCamera | None: ...
    def GetParentFrameId(self, context: pydrake.systems.framework.Context) -> pydrake.geometry.FrameId: ...
    def GetX_PB(self, context: pydrake.systems.framework.Context) -> pydrake.math.RigidTransform: ...
    def SetColorRenderCamera(self, context: pydrake.systems.framework.Context, color_camera: pydrake.geometry.ColorRenderCamera) -> None: ...
    def SetDepthRenderCamera(self, context: pydrake.systems.framework.Context, depth_camera: pydrake.geometry.DepthRenderCamera) -> None: ...
    def SetParentFrameId(self, context: pydrake.systems.framework.Context, id: pydrake.geometry.FrameId) -> None: ...
    def SetX_PB(self, context: pydrake.systems.framework.Context, sensor_pose: pydrake.math.RigidTransform) -> None: ...
    def body_pose_in_world_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def capture_offset(self) -> float: ...
    def color_image_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def default_X_PB(self) -> pydrake.math.RigidTransform: ...
    def default_color_render_camera(self) -> pydrake.geometry.ColorRenderCamera | None: ...
    def default_depth_render_camera(self) -> pydrake.geometry.DepthRenderCamera | None: ...
    def default_parent_frame_id(self) -> pydrake.geometry.FrameId: ...
    def depth_image_16U_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def depth_image_32F_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def fps(self) -> float: ...
    def image_time_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def label_image_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def output_delay(self) -> float: ...
    def set_default_X_PB(self, sensor_pose: pydrake.math.RigidTransform) -> None: ...
    def set_default_color_render_camera(self, color_camera: pydrake.geometry.ColorRenderCamera) -> None: ...
    def set_default_depth_render_camera(self, depth_camera: pydrake.geometry.DepthRenderCamera) -> None: ...
    def set_default_parent_frame_id(self, id: pydrake.geometry.FrameId) -> None: ...

class RgbdSensorDiscrete(pydrake.systems.framework.Diagram):
    kDefaultPeriod: ClassVar[float] = ...
    def __init__(self, sensor: RgbdSensor, period: float = ..., render_label_image: bool = ...) -> None: ...
    def body_pose_in_world_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def color_image_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def depth_image_16U_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def depth_image_32F_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def image_time_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def label_image_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def period(self) -> float: ...
    def query_object_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def sensor(self) -> RgbdSensor: ...

class RotaryEncoders(pydrake.systems.framework.VectorSystem):
    @overload
    def __init__(self, ticks_per_revolution: collections.abc.Sequence[int]) -> None: ...
    @overload
    def __init__(self, input_port_size: int, input_vector_indices: collections.abc.Sequence[int]) -> None: ...
    @overload
    def __init__(self, input_port_size: int, input_vector_indices: collections.abc.Sequence[int], ticks_per_revolution: collections.abc.Sequence[int]) -> None: ...
    def get_calibration_offsets(self, *args, **kwargs): ...
    def set_calibration_offsets(self, context: pydrake.systems.framework.Context, calibration_offsets) -> None: ...

class RotaryEncoders_𝓣AutoDiffXd𝓤(pydrake.systems.framework.VectorSystem_TAutoDiffXdU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, ticks_per_revolution: collections.abc.Sequence[int]) -> None: ...
    @overload
    def __init__(self, input_port_size: int, input_vector_indices: collections.abc.Sequence[int]) -> None: ...
    @overload
    def __init__(self, input_port_size: int, input_vector_indices: collections.abc.Sequence[int], ticks_per_revolution: collections.abc.Sequence[int]) -> None: ...
    def get_calibration_offsets(self, *args, **kwargs): ...
    def set_calibration_offsets(self, context: pydrake.systems.framework.Context_TAutoDiffXdU, calibration_offsets) -> None: ...

class RotaryEncoders_𝓣Expression𝓤(pydrake.systems.framework.VectorSystem_TExpressionU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, ticks_per_revolution: collections.abc.Sequence[int]) -> None: ...
    @overload
    def __init__(self, input_port_size: int, input_vector_indices: collections.abc.Sequence[int]) -> None: ...
    @overload
    def __init__(self, input_port_size: int, input_vector_indices: collections.abc.Sequence[int], ticks_per_revolution: collections.abc.Sequence[int]) -> None: ...
    def get_calibration_offsets(self, *args, **kwargs): ...
    def set_calibration_offsets(self, context: pydrake.systems.framework.Context_TExpressionU, calibration_offsets) -> None: ...

class _TemporaryName_N5drake7systems7sensors11ImageTraitsILNS1_9PixelTypeE0EEE:
    ChannelType: ClassVar[type[numpy.uint8]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class _TemporaryName_N5drake7systems7sensors11ImageTraitsILNS1_9PixelTypeE1EEE:
    ChannelType: ClassVar[type[numpy.uint8]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class _TemporaryName_N5drake7systems7sensors11ImageTraitsILNS1_9PixelTypeE2EEE:
    ChannelType: ClassVar[type[numpy.uint8]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class _TemporaryName_N5drake7systems7sensors11ImageTraitsILNS1_9PixelTypeE3EEE:
    ChannelType: ClassVar[type[numpy.uint8]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class _TemporaryName_N5drake7systems7sensors11ImageTraitsILNS1_9PixelTypeE4EEE:
    ChannelType: ClassVar[type[numpy.uint8]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class _TemporaryName_N5drake7systems7sensors11ImageTraitsILNS1_9PixelTypeE5EEE:
    ChannelType: ClassVar[type[numpy.uint16]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class _TemporaryName_N5drake7systems7sensors11ImageTraitsILNS1_9PixelTypeE6EEE:
    ChannelType: ClassVar[type[numpy.float32]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class _TemporaryName_N5drake7systems7sensors11ImageTraitsILNS1_9PixelTypeE7EEE:
    ChannelType: ClassVar[type[numpy.int16]] = ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    kNumChannels: ClassVar[int] = ...
    kPixelFormat: ClassVar[PixelFormat] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class _TemporaryName_N5drake7systems7sensors13AccelerometerINS_2ad8AutoDiffEEE(pydrake.systems.framework.LeafSystem_TAutoDiffXdU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, body: pydrake.multibody.tree.RigidBody_TAutoDiffXdU, X_BS: pydrake.math.RigidTransform, gravity_vector=...) -> None: ...
    def body_index(self) -> pydrake.multibody.tree.BodyIndex: ...
    def get_body_accelerations_input_port(self) -> pydrake.systems.framework.InputPort_TAutoDiffXdU: ...
    def get_body_poses_input_port(self) -> pydrake.systems.framework.InputPort_TAutoDiffXdU: ...
    def get_body_velocities_input_port(self) -> pydrake.systems.framework.InputPort_TAutoDiffXdU: ...
    def get_measurement_output_port(self) -> pydrake.systems.framework.OutputPort_TAutoDiffXdU: ...
    def gravity_vector(self, *args, **kwargs): ...
    def pose(self) -> pydrake.math.RigidTransform: ...

class _TemporaryName_N5drake7systems7sensors13AccelerometerINS_8symbolic10ExpressionEEE(pydrake.systems.framework.LeafSystem_TExpressionU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, body: pydrake.multibody.tree.RigidBody_TExpressionU, X_BS: pydrake.math.RigidTransform, gravity_vector=...) -> None: ...
    def body_index(self) -> pydrake.multibody.tree.BodyIndex: ...
    def get_body_accelerations_input_port(self) -> pydrake.systems.framework.InputPort_TExpressionU: ...
    def get_body_poses_input_port(self) -> pydrake.systems.framework.InputPort_TExpressionU: ...
    def get_body_velocities_input_port(self) -> pydrake.systems.framework.InputPort_TExpressionU: ...
    def get_measurement_output_port(self) -> pydrake.systems.framework.OutputPort_TExpressionU: ...
    def gravity_vector(self, *args, **kwargs): ...
    def pose(self) -> pydrake.math.RigidTransform: ...

class _TemporaryName_N5drake7systems7sensors14RotaryEncodersINS_2ad8AutoDiffEEE(pydrake.systems.framework.VectorSystem_TAutoDiffXdU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, ticks_per_revolution: collections.abc.Sequence[int]) -> None: ...
    @overload
    def __init__(self, input_port_size: int, input_vector_indices: collections.abc.Sequence[int]) -> None: ...
    @overload
    def __init__(self, input_port_size: int, input_vector_indices: collections.abc.Sequence[int], ticks_per_revolution: collections.abc.Sequence[int]) -> None: ...
    def get_calibration_offsets(self, *args, **kwargs): ...
    def set_calibration_offsets(self, context: pydrake.systems.framework.Context_TAutoDiffXdU, calibration_offsets) -> None: ...

class _TemporaryName_N5drake7systems7sensors14RotaryEncodersINS_8symbolic10ExpressionEEE(pydrake.systems.framework.VectorSystem_TExpressionU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self, ticks_per_revolution: collections.abc.Sequence[int]) -> None: ...
    @overload
    def __init__(self, input_port_size: int, input_vector_indices: collections.abc.Sequence[int]) -> None: ...
    @overload
    def __init__(self, input_port_size: int, input_vector_indices: collections.abc.Sequence[int], ticks_per_revolution: collections.abc.Sequence[int]) -> None: ...
    def get_calibration_offsets(self, *args, **kwargs): ...
    def set_calibration_offsets(self, context: pydrake.systems.framework.Context_TExpressionU, calibration_offsets) -> None: ...

class _TemporaryName_N5drake7systems7sensors5ImageILNS1_9PixelTypeE0EEE:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class _TemporaryName_N5drake7systems7sensors5ImageILNS1_9PixelTypeE1EEE:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class _TemporaryName_N5drake7systems7sensors5ImageILNS1_9PixelTypeE2EEE:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class _TemporaryName_N5drake7systems7sensors5ImageILNS1_9PixelTypeE3EEE:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class _TemporaryName_N5drake7systems7sensors5ImageILNS1_9PixelTypeE4EEE:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint8]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class _TemporaryName_N5drake7systems7sensors5ImageILNS1_9PixelTypeE5EEE:
    class Traits:
        ChannelType: ClassVar[type[numpy.uint16]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class _TemporaryName_N5drake7systems7sensors5ImageILNS1_9PixelTypeE6EEE:
    class Traits:
        ChannelType: ClassVar[type[numpy.float32]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: float) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class _TemporaryName_N5drake7systems7sensors5ImageILNS1_9PixelTypeE7EEE:
    class Traits:
        ChannelType: ClassVar[type[numpy.int16]] = ...
        _original_name: ClassVar[str] = ...
        _original_qualname: ClassVar[str] = ...
        kNumChannels: ClassVar[int] = ...
        kPixelFormat: ClassVar[PixelFormat] = ...
        def __init__(self, *args, **kwargs) -> None: ...
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, width: int, height: int) -> None: ...
    @overload
    def __init__(self, width: int, height: int, initial_value: int) -> None: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    @overload
    def at(self, x, y) -> Any: ...
    def height(self) -> int: ...
    def resize(self, arg0: int, arg1: int) -> None: ...
    def size(self) -> int: ...
    def width(self) -> int: ...
    @property
    def data(self): ...
    @property
    def mutable_data(self): ...
    @property
    def shape(self): ...

class _TemporaryName_N5drake7systems7sensors9GyroscopeINS_2ad8AutoDiffEEE(pydrake.systems.framework.LeafSystem_TAutoDiffXdU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, body: pydrake.multibody.tree.RigidBody_TAutoDiffXdU, X_BS: pydrake.math.RigidTransform) -> None: ...
    def body_index(self) -> pydrake.multibody.tree.BodyIndex: ...
    def get_body_poses_input_port(self) -> pydrake.systems.framework.InputPort_TAutoDiffXdU: ...
    def get_body_velocities_input_port(self) -> pydrake.systems.framework.InputPort_TAutoDiffXdU: ...
    def get_measurement_output_port(self) -> pydrake.systems.framework.OutputPort_TAutoDiffXdU: ...
    def pose(self) -> pydrake.math.RigidTransform: ...

class _TemporaryName_N5drake7systems7sensors9GyroscopeINS_8symbolic10ExpressionEEE(pydrake.systems.framework.LeafSystem_TExpressionU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, body: pydrake.multibody.tree.RigidBody_TExpressionU, X_BS: pydrake.math.RigidTransform) -> None: ...
    def body_index(self) -> pydrake.multibody.tree.BodyIndex: ...
    def get_body_poses_input_port(self) -> pydrake.systems.framework.InputPort_TExpressionU: ...
    def get_body_velocities_input_port(self) -> pydrake.systems.framework.InputPort_TExpressionU: ...
    def get_measurement_output_port(self) -> pydrake.systems.framework.OutputPort_TExpressionU: ...
    def pose(self) -> pydrake.math.RigidTransform: ...



import enum
import nanobind
from . import eigen_geometry as eigen_geometry, schema as schema, value as value
from typing import Any, Callable, ClassVar, overload

CalcProbabilityDensity: nanobind.nb_func
FindResourceOrThrow: nanobind.nb_func
GetDrakePath: nanobind.nb_func
configure_logging: Callable
kDrakeAssertIsArmed: bool
pretty_class_name: Callable
temp_directory: nanobind.nb_func
trigger_an_assertion_failure: nanobind.nb_func
use_native_cpp_logging: Callable

class MemoryFile:
    Make: ClassVar[nanobind.nb_func] = ...
    _rewrite_yaml_dump_attr_name: ClassVar[nanobind.nb_func] = ...
    def __init__(self, contents: bytes | str = ..., extension: str = ..., filename_hint: str = ...) -> None: ...
    def contents(self) -> bytes: ...
    @overload
    def extension(self) -> str: ...
    @overload
    def extension(self, aspassedtotheconstructor) -> Any: ...
    def filename_hint(self) -> str: ...
    def sha256(self) -> Sha256: ...
    def __copy__(self) -> MemoryFile: ...
    def __deepcopy__(self, arg: dict) -> MemoryFile: ...
    def __setattr__(self, arg0: str, arg1: object) -> None: ...
    @property
    def __fields__(self): ...

class Parallelism:
    Max: ClassVar[nanobind.nb_func] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, parallelize: bool) -> None: ...
    @overload
    def __init__(self, num_threads: int) -> None: ...
    def num_threads(self) -> int: ...
    def __copy__(self) -> Parallelism: ...
    def __deepcopy__(self, arg: dict) -> Parallelism: ...

class RandomDistribution(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kExponential: ClassVar[RandomDistribution] = ...
    kGaussian: ClassVar[RandomDistribution] = ...
    kUniform: ClassVar[RandomDistribution] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class RandomGenerator:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, seed: int) -> None: ...
    def __call__(self) -> int: ...

class Sha256:
    Checksum: ClassVar[nanobind.nb_func] = ...
    Parse: ClassVar[nanobind.nb_func] = ...
    def __init__(self) -> None: ...
    def to_string(self) -> str: ...
    def __copy__(self) -> Sha256: ...
    def __deepcopy__(self, arg: dict) -> Sha256: ...
    def __eq__(self, arg: Sha256) -> bool: ...
    def __lt__(self, arg: Sha256) -> bool: ...
    def __ne__(self, arg: Sha256) -> bool: ...

class ToleranceType(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kAbsolute: ClassVar[ToleranceType] = ...
    kRelative: ClassVar[ToleranceType] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class _MangledName:
    UNICODE_COMMA: ClassVar[str] = ...
    UNICODE_LEFT_BRACKET: ClassVar[str] = ...
    UNICODE_PERIOD: ClassVar[str] = ...
    UNICODE_RIGHT_BRACKET: ClassVar[str] = ...
    demangle: ClassVar[Callable] = ...
    mangle: ClassVar[Callable] = ...
    module_getattr: ClassVar[Callable] = ...



import collections.abc
import nanobind
from _typeshed import Incomplete
from typing import Any, Callable, ClassVar, overload

class DrakeLcm(DrakeLcmInterface):
    available: ClassVar[nanobind.nb_func] = ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, lcm_url: str) -> None: ...
    @overload
    def __init__(self, params: DrakeLcmParams) -> None: ...

class DrakeLcmInterface:
    def __init__(self, *args, **kwargs) -> None: ...
    def HandleSubscriptions(self, timeout_millis: int) -> int: ...
    def Publish(self, channel: str, buffer: bytes, time_sec: float | None = ...) -> None: ...
    def Subscribe(self, channel: str, handler: collections.abc.Callable[[bytes], None]) -> None: ...
    def SubscribeAllChannels(self, handler: collections.abc.Callable[[str, bytes], None]) -> None: ...
    def SubscribeMultichannel(self, regex: str, handler: collections.abc.Callable[[str, bytes], None]) -> None: ...
    def get_lcm_url(self) -> str: ...

class DrakeLcmParams:
    channel_suffix: Incomplete
    defer_initialization: Any
    lcm_url: str
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> DrakeLcmParams: ...
    def __deepcopy__(self, arg: dict) -> DrakeLcmParams: ...
    @property
    def __fields__(self): ...

class Subscriber:
    __init__: ClassVar[Callable] = ...
    _handler: ClassVar[Callable] = ...
    clear: ClassVar[Callable] = ...



"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""


from io import BytesIO
import struct

class lcmt_viewer_draw(object):

    __slots__ = ["timestamp", "num_links", "link_name", "robot_num", "position", "quaternion"]

    __typenames__ = ["int64_t", "int32_t", "string", "int32_t", "float", "float"]

    __dimensions__ = [None, None, ["num_links"], ["num_links"], ["num_links", 3], ["num_links", 4]]

    def __init__(self):
        self.timestamp = 0
        """
        The timestamp in milliseconds.
        LCM Type: int64_t
        """

        self.num_links = 0
        """ LCM Type: int32_t """
        self.link_name = []
        """ LCM Type: string[num_links] """
        self.robot_num = []
        """ LCM Type: int32_t[num_links] """
        self.position = []
        """ LCM Type: float[num_links][3] """
        self.quaternion = []
        """
        x, y, z
        LCM Type: float[num_links][4]
        """


    def encode(self):
        buf = BytesIO()
        buf.write(lcmt_viewer_draw._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        buf.write(struct.pack(">qi", self.timestamp, self.num_links))
        for i0 in range(self.num_links):
            __link_name_encoded = self.link_name[i0].encode('utf-8')
            buf.write(struct.pack('>I', len(__link_name_encoded)+1))
            buf.write(__link_name_encoded)
            buf.write(b"\0")
        buf.write(struct.pack('>%di' % self.num_links, *self.robot_num[:self.num_links]))
        for i0 in range(self.num_links):
            buf.write(struct.pack('>3f', *self.position[i0][:3]))
        for i0 in range(self.num_links):
            buf.write(struct.pack('>4f', *self.quaternion[i0][:4]))

    @staticmethod
    def decode(data: bytes):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != lcmt_viewer_draw._get_packed_fingerprint():
            raise ValueError("Decode error")
        return lcmt_viewer_draw._decode_one(buf)

    @staticmethod
    def _decode_one(buf):
        self = lcmt_viewer_draw()
        self.timestamp, self.num_links = struct.unpack(">qi", buf.read(12))
        self.link_name = []
        for i0 in range(self.num_links):
            __link_name_len = struct.unpack('>I', buf.read(4))[0]
            self.link_name.append(buf.read(__link_name_len)[:-1].decode('utf-8', 'replace'))
        self.robot_num = struct.unpack('>%di' % self.num_links, buf.read(self.num_links * 4))
        self.position = []
        for i0 in range(self.num_links):
            self.position.append(struct.unpack('>3f', buf.read(12)))
        self.quaternion = []
        for i0 in range(self.num_links):
            self.quaternion.append(struct.unpack('>4f', buf.read(16)))
        return self

    @staticmethod
    def _get_hash_recursive(parents):
        if lcmt_viewer_draw in parents: return 0
        tmphash = (0x20a785ff2d97a122) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _packed_fingerprint = None

    @staticmethod
    def _get_packed_fingerprint():
        if lcmt_viewer_draw._packed_fingerprint is None:
            lcmt_viewer_draw._packed_fingerprint = struct.pack(">Q", lcmt_viewer_draw._get_hash_recursive([]))
        return lcmt_viewer_draw._packed_fingerprint

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", lcmt_viewer_draw._get_packed_fingerprint())[0]




"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""


from io import BytesIO
import struct

class lcmt_viewer_command(object):

    __slots__ = ["command_type", "command_data"]

    __typenames__ = ["int8_t", "string"]

    __dimensions__ = [None, None]

    STATUS = 0
    """ enum for viewer command type """
    LOAD_MODEL = 1
    LOAD_RENDERER = 2
    SHUTDOWN = 3
    START_RECORDING = 4
    STOP_RECORDING = 5
    LOAD_TERRAIN = 6
    SET_TERRAIN_TRANSFORM = 7

    def __init__(self):
        self.command_type = 0
        """ LCM Type: int8_t """
        self.command_data = ""
        """ LCM Type: string """

    def encode(self):
        buf = BytesIO()
        buf.write(lcmt_viewer_command._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        buf.write(struct.pack(">b", self.command_type))
        __command_data_encoded = self.command_data.encode('utf-8')
        buf.write(struct.pack('>I', len(__command_data_encoded)+1))
        buf.write(__command_data_encoded)
        buf.write(b"\0")

    @staticmethod
    def decode(data: bytes):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != lcmt_viewer_command._get_packed_fingerprint():
            raise ValueError("Decode error")
        return lcmt_viewer_command._decode_one(buf)

    @staticmethod
    def _decode_one(buf):
        self = lcmt_viewer_command()
        self.command_type = struct.unpack(">b", buf.read(1))[0]
        __command_data_len = struct.unpack('>I', buf.read(4))[0]
        self.command_data = buf.read(__command_data_len)[:-1].decode('utf-8', 'replace')
        return self

    @staticmethod
    def _get_hash_recursive(parents):
        if lcmt_viewer_command in parents: return 0
        tmphash = (0x7878fb2792b4a897) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _packed_fingerprint = None

    @staticmethod
    def _get_packed_fingerprint():
        if lcmt_viewer_command._packed_fingerprint is None:
            lcmt_viewer_command._packed_fingerprint = struct.pack(">Q", lcmt_viewer_command._get_hash_recursive([]))
        return lcmt_viewer_command._packed_fingerprint

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", lcmt_viewer_command._get_packed_fingerprint())[0]




import enum
import nanobind
import pydrake.multibody.plant
import pydrake.systems.framework
import types
from _typeshed import Incomplete
from typing import Callable, ClassVar

ApplyDriverConfig: nanobind.nb_func
ApplyDriverConfigs: Callable
ApplyNamedPositionsAsDefaults: nanobind.nb_func
BuildIiwaControl: nanobind.nb_func
BuildSchunkWsgControl: nanobind.nb_func
GetSchunkWsgOpenPosition: nanobind.nb_func
MakeMultibodyForceToWsgForceSystem: nanobind.nb_func
MakeMultibodyStateToWsgStateSystem: nanobind.nb_func
PandaControlMode: types.SimpleNamespace
ParseIiwaControlMode: nanobind.nb_func
get_iiwa_max_joint_velocities: nanobind.nb_func
kIiwaArmNumJoints: int
kIiwaLcmStatusPeriod: float
kPandaArmNumJoints: int
position_enabled: nanobind.nb_func
torque_enabled: nanobind.nb_func

class IiwaCommandReceiver(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int = ..., control_mode: IiwaControlMode = ...) -> None: ...
    def get_commanded_position_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_commanded_torque_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_message_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_position_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_time_output_port(self) -> pydrake.systems.framework.OutputPort: ...

class IiwaCommandSender(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int = ..., control_mode: IiwaControlMode = ...) -> None: ...
    def get_position_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_time_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_input_port(self) -> pydrake.systems.framework.InputPort: ...

class IiwaControlMode(enum.Enum):
    __new__: ClassVar[Callable] = ...
    _generate_next_value_: ClassVar[Callable] = ...
    _hashable_values_: ClassVar[list] = ...
    _member_map_: ClassVar[dict] = ...
    _member_names_: ClassVar[list] = ...
    _member_type_: ClassVar[type[object]] = ...
    _unhashable_values_: ClassVar[list] = ...
    _unhashable_values_map_: ClassVar[dict] = ...
    _use_args_: ClassVar[bool] = ...
    _value2member_map_: ClassVar[dict] = ...
    _value_repr_: ClassVar[None] = ...
    kPositionAndTorque: ClassVar[IiwaControlMode] = ...
    kPositionOnly: ClassVar[IiwaControlMode] = ...
    kTorqueOnly: ClassVar[IiwaControlMode] = ...
    __nb_enum__: ClassVar[PyCapsule] = ...

class IiwaDriver:
    arm_child_frame_name: str | None
    control_mode: str
    desired_kp_gains: Incomplete
    ext_joint_filter_tau: float
    gripper_parent_frame_name: str | None
    hand_model_name: str
    lcm_bus: Incomplete
    lcm_status_period: float
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> IiwaDriver: ...
    def __deepcopy__(self, arg: dict) -> IiwaDriver: ...
    @property
    def __fields__(self): ...

class IiwaStatusReceiver(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int = ...) -> None: ...
    def get_position_commanded_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_position_measured_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_time_measured_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_torque_commanded_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_torque_external_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_torque_measured_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_velocity_estimated_output_port(self) -> pydrake.systems.framework.OutputPort: ...

class IiwaStatusSender(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int = ...) -> None: ...
    def get_position_commanded_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_position_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_time_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_commanded_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_external_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_velocity_estimated_input_port(self) -> pydrake.systems.framework.InputPort: ...

class PandaCommandReceiver(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int, control_mode: int) -> None: ...
    def LatchInitialPosition(self, context: pydrake.systems.framework.Context) -> None: ...
    def get_commanded_position_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_commanded_torque_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_commanded_velocity_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_message_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_position_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...

class PandaCommandSender(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int, control_mode: int) -> None: ...
    def get_position_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_velocity_input_port(self) -> pydrake.systems.framework.InputPort: ...

class PandaStatusReceiver(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int = ...) -> None: ...
    def get_acceleration_commanded_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_position_commanded_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_position_measured_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_torque_commanded_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_torque_external_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_torque_measured_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_velocity_commanded_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_velocity_measured_output_port(self) -> pydrake.systems.framework.OutputPort: ...

class PandaStatusSender(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int = ...) -> None: ...
    def get_acceleration_commanded_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_position_commanded_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_position_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_commanded_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_external_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_velocity_commanded_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_velocity_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...

class SchunkWsgCommandReceiver(pydrake.systems.framework.LeafSystem):
    def __init__(self, initial_position: float = ..., initial_force: float = ...) -> None: ...
    def get_force_limit_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_position_output_port(self) -> pydrake.systems.framework.OutputPort: ...

class SchunkWsgCommandSender(pydrake.systems.framework.LeafSystem):
    def __init__(self, default_force_limit: float = ...) -> None: ...
    def get_command_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_force_limit_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_position_input_port(self) -> pydrake.systems.framework.InputPort: ...

class SchunkWsgController(pydrake.systems.framework.Diagram):
    def __init__(self, kp: float = ..., ki: float = ..., kd: float = ...) -> None: ...

class SchunkWsgDriver:
    lcm_bus: Incomplete
    pid_gains: Incomplete
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> SchunkWsgDriver: ...
    def __deepcopy__(self, arg: dict) -> SchunkWsgDriver: ...
    @property
    def __fields__(self): ...

class SchunkWsgPositionController(pydrake.systems.framework.Diagram):
    def __init__(self, time_step: float = ..., kp_command: float = ..., kd_command: float = ..., kp_constraint: float = ..., kd_constraint: float = ..., default_force_limit: float = ...) -> None: ...
    def get_desired_position_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_force_limit_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_generalized_force_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_grip_force_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_state_input_port(self) -> pydrake.systems.framework.InputPort: ...

class SchunkWsgStatusReceiver(pydrake.systems.framework.LeafSystem):
    def __init__(self) -> None: ...
    def get_force_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_state_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_status_input_port(self) -> pydrake.systems.framework.InputPort: ...

class SchunkWsgStatusSender(pydrake.systems.framework.LeafSystem):
    def __init__(self) -> None: ...
    def get_force_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_state_input_port(self) -> pydrake.systems.framework.InputPort: ...

class SchunkWsgTrajectoryGenerator(pydrake.systems.framework.LeafSystem):
    def __init__(self, input_size: int, position_index: int, use_force_limit: bool = ...) -> None: ...
    def get_desired_position_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_force_limit_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_max_force_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_state_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_target_output_port(self) -> pydrake.systems.framework.OutputPort: ...

class SimIiwaDriver(pydrake.systems.framework.Diagram):
    AddToBuilder: ClassVar[nanobind.nb_func] = ...
    def __init__(self, driver_config: IiwaDriver, controller_plant: pydrake.multibody.plant.MultibodyPlant) -> None: ...

class ZeroForceDriver:
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> ZeroForceDriver: ...
    def __deepcopy__(self, arg: dict) -> ZeroForceDriver: ...
    @property
    def __fields__(self): ...
